@@ -29,6 +29,7 @@
 #include <libxml/HTMLparser.h>
 #include <string.h>
 #include <net/grl-net.h>
+#include <glib/gi18n-lib.h>
 
 #include "grl-raitv.h"
 
@@ -44,11 +45,11 @@ GRL_LOG_DOMAIN_STATIC(raitv_log_domain);
 #define ROOT_DIR_POPULARS_INDEX	0
 #define ROOT_DIR_RECENTS_INDEX 	1
 
-#define RAITV_POPULARS_ID	"most-popular"
-#define RAITV_POPULARS_NAME	"Most Popular"
+#define RAITV_POPULARS_ID	 "most-popular"
+#define RAITV_POPULARS_NAME N_("Most Popular")
 
 #define RAITV_RECENTS_ID	"recent"
-#define RAITV_RECENTS_NAME	"Recent"
+#define RAITV_RECENTS_NAME	N_("Recent")
 
 #define RAITV_POPULARS_THEME_ID	"theme-popular"
 #define RAITV_RECENTS_THEME_ID	"theme-recent"
@@ -91,7 +92,7 @@ GRL_LOG_DOMAIN_STATIC(raitv_log_domain);
 
 #define SOURCE_ID   "grl-raitv"
 #define SOURCE_NAME "Rai.tv"
-#define SOURCE_DESC "A source for browsing and searching Rai.tv videos"
+#define SOURCE_DESC _("A source for browsing and searching Rai.tv videos")
 
 
 G_DEFINE_TYPE (GrlRaitvSource, grl_raitv_source, GRL_TYPE_SOURCE)
@@ -169,29 +170,29 @@ CategoryInfo root_dir[] = {
 };
 
 CategoryInfo themes_dir[] = {
-  {"all","All",-1,"","Tematica:News^Tematica:ntz"},
-  {"bianco_nero","Bianco e Nero",-1,"Tematica:Bianco e Nero",""},
-  {"cinema","Cinema",-1,"Tematica:Cinema",""},
-  {"comici","Comici",-1,"Tematica:Comici",""},
-  {"cronaca","Cronaca",-1,"Tematica:Cronaca",""},
-  {"cultura","Cultura",-1,"Tematica:Cultura",""},
-  {"economia","Economia",-1,"Tematica:Economia",""},
-  {"fiction","Fiction",-1,"Tematica:Fiction",""},
-  {"junior","Junior",-1,"Tematica:Junior",""},
-  {"inchieste","Inchieste",-1,"Tematica:Inchieste",""},
-  {"interviste","Interviste",-1,"Tematica:Interviste",""},
-  {"musica","Musica",-1,"Tematica:Musica",""},
-  {"news","News",-1,"Tematica:News",""},
-  {"salute","Salute",-1,"Tematica:Salute",""},
-  {"satira","Satira",-1,"Tematica:Satira",""},
-  {"scienza","Scienza",-1,"Tematica:Scienza",""},
-  {"societa","Societa",-1,"Tematica:Societa",""},
-  {"spettacolo","Spettacolo",-1,"Tematica:Spettacolo",""},
-  {"sport","Sport",-1,"Tematica:Sport",""},
-  {"storia","Storia",-1,"Tematica:Storia",""},
-  {"politica","Politica",-1,"Tematica:Politica",""},
-  {"tempo_libero","Tempo libero",-1,"Tematica:Tempo libero",""},
-  {"viaggi","Viaggi",-1,"Tematica:Viaggi",""},
+  {"all",N_("All"),-1,"","Tematica:News^Tematica:ntz"},
+  {"bianco_nero",N_("Black and White"),-1,"Tematica:Bianco e Nero",""},
+  {"cinema",N_("Cinema"),-1,"Tematica:Cinema",""},
+  {"comici",N_("Comedians"),-1,"Tematica:Comici",""},
+  {"cronaca",N_("Chronicle"),-1,"Tematica:Cronaca",""},
+  {"cultura",N_("Culture"),-1,"Tematica:Cultura",""},
+  {"economia",N_("Economy"),-1,"Tematica:Economia",""},
+  {"fiction",N_("Fiction"),-1,"Tematica:Fiction",""},
+  {"junior",N_("Junior"),-1,"Tematica:Junior",""},
+  {"inchieste",N_("Investigations"),-1,"Tematica:Inchieste",""},
+  {"interviste",N_("Interviews"),-1,"Tematica:Interviste",""},
+  {"musica",N_("Music"),-1,"Tematica:Musica",""},
+  {"news",N_("News"),-1,"Tematica:News",""},
+  {"salute",N_("Health"),-1,"Tematica:Salute",""},
+  {"satira",N_("Satire"),-1,"Tematica:Satira",""},
+  {"scienza","Science",-1,"Tematica:Scienza",""},
+  {"societa",N_("Society"),-1,"Tematica:Societa",""},
+  {"spettacolo",N_("Show"),-1,"Tematica:Spettacolo",""},
+  {"sport",N_("Sport"),-1,"Tematica:Sport",""},
+  {"storia",N_("History"),-1,"Tematica:Storia",""},
+  {"politica",N_("Politics"),-1,"Tematica:Politica",""},
+  {"tempo_libero",N_("Leisure"),-1,"Tematica:Tempo libero",""},
+  {"viaggi",N_("Travel"),-1,"Tematica:Viaggi",""},
   {NULL, NULL, 0}
 };
 
@@ -236,6 +237,10 @@ grl_raitv_plugin_init (GrlRegistry *registry,
 {
   GRL_LOG_DOMAIN_INIT (raitv_log_domain, "raitv");
 
+  /* Initialize i18n */
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
   GrlRaitvSource *source = grl_raitv_source_new ();
   grl_registry_register_source (registry,
                                 plugin,
@@ -279,12 +284,12 @@ grl_raitv_source_finalize (GObject *object)
   }
 
   if (source->priv->raitv_search_mappings != NULL) {
-    g_object_unref (source->priv->raitv_search_mappings);
+    g_list_free_full (source->priv->raitv_search_mappings, g_free);
     source->priv->raitv_search_mappings = NULL;
   }
 
   if (source->priv->raitv_browse_mappings != NULL) {
-    g_object_unref (source->priv->raitv_browse_mappings);
+    g_list_free_full (source->priv->raitv_browse_mappings, g_free);
     source->priv->raitv_browse_mappings = NULL;
   }
 
@@ -387,7 +392,7 @@ proxy_call_search_grlnet_async_cb (GObject *source_object,
   xmlXPathContextPtr  xpath = NULL;
   xmlXPathObjectPtr   obj = NULL;
   gint i, nb_items = 0;
-  gint length;
+  gsize length;
 
   GRL_DEBUG ("Response id=%u", op->operation_id);
 
@@ -403,11 +408,11 @@ proxy_call_search_grlnet_async_cb (GObject *source_object,
   if (!grl_net_wc_request_finish (GRL_NET_WC (source_object),
                                   res,
                                   &content,
-                                  (gsize *) &length,
+                                  &length,
                                   &wc_error)) {
     error = g_error_new (GRL_CORE_ERROR,
                          GRL_CORE_ERROR_SEARCH_FAILED,
-                         "Failed to search Rait.tv: '%s'",
+                         _("Failed to search: %s"),
                          wc_error->message);
 
     op->callback (op->source,
@@ -423,7 +428,7 @@ proxy_call_search_grlnet_async_cb (GObject *source_object,
     return;
   }
 
-  doc = xmlParseMemory (content, length);
+  doc = xmlParseMemory (content, (gint) length);
 
   if (!doc) {
     GRL_DEBUG ("Doc failed");
@@ -583,7 +588,7 @@ proxy_call_browse_grlnet_async_cb (GObject *source_object,
   xmlXPathContextPtr  xpath = NULL;
   xmlXPathObjectPtr   obj = NULL;
   gint i, nb_items = 0;
-  gint length;
+  gsize length;
 
 
   GRL_DEBUG ("Response id=%u", op->operation_id);
@@ -600,11 +605,11 @@ proxy_call_browse_grlnet_async_cb (GObject *source_object,
   if (!grl_net_wc_request_finish (GRL_NET_WC (source_object),
                                   res,
                                   &content,
-                                  (gsize *) &length,
+                                  &length,
                                   &wc_error)) {
     error = g_error_new (GRL_CORE_ERROR,
                          GRL_CORE_ERROR_SEARCH_FAILED,
-                         "Failed to browse Rait.tv: '%s'",
+                         _("Failed to browse: %s"),
                          wc_error->message);
 
     op->callback (op->source,
@@ -620,7 +625,7 @@ proxy_call_browse_grlnet_async_cb (GObject *source_object,
     return;
   }
 
-  doc = xmlRecoverMemory (content, length);
+  doc = xmlRecoverMemory (content, (gint) length);
 
   if (!doc) {
     GRL_DEBUG ("Doc failed");
@@ -838,7 +843,7 @@ proxy_call_resolve_grlnet_async_cb (GObject *source_object,
   GError             *wc_error = NULL;
   GError             *error = NULL;
   gchar              *content = NULL;
-  gint                length;
+  gsize               length;
   gchar              *value;
   gchar              *thumbnail;
   gchar             **tokens;
@@ -854,11 +859,11 @@ proxy_call_resolve_grlnet_async_cb (GObject *source_object,
   if (!grl_net_wc_request_finish (GRL_NET_WC (source_object),
                                   res,
                                   &content,
-                                  (gsize *) &length,
+                                  &length,
                                   &wc_error)) {
     error = g_error_new (GRL_CORE_ERROR,
                          GRL_CORE_ERROR_SEARCH_FAILED,
-                         "Failed to resolve Rait.tv: '%s'",
+                         _("Failed to resolve: %s"),
                          wc_error->message);
 
     op->resolveCb (op->source,
@@ -873,7 +878,7 @@ proxy_call_resolve_grlnet_async_cb (GObject *source_object,
     return;
   }
 
-  doc = xmlRecoverMemory (content, length);
+  doc = xmlRecoverMemory (content, (gint) length);
 
   if (!doc) {
     GRL_DEBUG ("Doc failed");
@@ -1031,7 +1036,7 @@ produce_container_from_directory (GrlMedia *media,
     GRL_DEBUG ("MediaId=%s, Type:%d, Titolo:%s",mediaid, type, dir[index].name);
 
     grl_media_set_id (content, mediaid);
-    grl_media_set_title (content, dir[index].name);
+    grl_media_set_title (content, g_dgettext (GETTEXT_PACKAGE, dir[index].name));
     g_free(mediaid);
   }
 
