@@ -27,8 +27,6 @@
 
 #define TMDB_PLUGIN_ID "grl-tmdb"
 
-const char *iso_date (const GDateTime *date, char **strbuf);
-
 /** Compare the floats.
  * A simple == will fail on values that are effectively the same,
  * due to rounding issues.
@@ -40,14 +38,6 @@ compare_floats(gfloat a, gfloat b)
   return fabs(a - b) < DBL_EPSILON;
 }
 
-const char *iso_date (const GDateTime *date, char **strbuf)
-{
-  if (*strbuf)
-    g_free (*strbuf);
-
-  return (*strbuf = g_date_time_format ((GDateTime *) date, "%F"));
-}
-
 static const gchar*
 get_region_certificate (GrlMedia *media, const gchar *region)
 {
